package client;

import java.awt.*;
import java.awt.event.*;
import java.util.*; // Observer & Observable

import tictactoe.GameFactory;
import tictactoe.RemoteGame;
import tictactoe.Move;

/**
 * This class implements the view on a TicTacToe game.
 * The Applet contains a grid of Place objects that
 * intercept mouse clicks and forward them to the game's
 * Players.  The Applet is also an Observer for the game.
 * When the game notifies the Applet of state change, it
 * updates the corresponding Place.  Below the grid there
 * is a Label component to display messages.
 * <P>
 * <B>To do:</B>
 * <UL>
 * <LI>add a mechanism to select TicTacToe or Gomoku
 * <LI>visualize automated tests?
 * </UL>
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */
public class GameView extends Frame implements Observer
{
	private RemoteGame game_;
	private Label label_;
	private Place[][] places_;
	private char mark_;

	static private final int MINSIZE = 30;

	/**
	 * Instantiates the RemoteGame, initializes the view components,
	 * and connects the view to the model.
	 */
	public GameView() {
		game_ = GameFactory.joinGame();
		mark_ = game_.join();
		game_.addObserver(this);
		
		this.setTitle("Player " + mark());

		setLayout(new BorderLayout());
		setSize(MINSIZE*game_.cols(),MINSIZE*game_.rows());

		this.add("North", makeControls());

		add("Center", makeGrid());
		
		label_ = new Label();
		add("South", label_);

		showFeedBack(game_.currentPlayer().mark() + " plays");
	}

	/**
	 * @return the RemoteGame associated with this view
	 */
	public RemoteGame game() { return game_; }

	/**
	 * @return the mark associated with this view
	 */
	public char mark() { return mark_; }

	private Component makeControls() {
		Panel controls = new Panel();
		controls.setLayout(new FlowLayout());

		Button again = new Button("New game");
		again.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showFeedBack("starting new game ...");
				GameView view = new GameView();
				view.show();
				dispose();
			}
		});
		controls.add(again);

		Button quit = new Button("Quit");
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showFeedBack("Goodbye!");
				dispose();
			}
		});
		controls.add(quit);

		return controls;
	}

	/**
	 * Make the grid that serves as a view and controller for the game.
	 */
	private Component makeGrid() {
		int cols = game_.cols();
		int rows = game_.rows();

		Panel grid = new Panel();
		
		grid.setLayout(new GridLayout(rows, cols));

		places_ = new Place[cols][rows];
		for (int row=rows-1; row>=0; row--) {
			for (int col=0; col<cols; col++) {
				Place p = new Place(col, row);
				p.addMouseListener(new PlaceListener(p, this));
				p.setBackground(Color.white);
				grid.add(p);
				places_[col][row] = p;
			}
		}
		return grid;
	}

	/**
	 * Displays a feedback string in the associated Label component.
	 * @param msg the string to display
	 */
	public void showFeedBack(String msg) {
		label_.setText(msg);
	}

	/**
	 * Implements the java.util.Observer interface.
	 * Called by the BoardGame when its state changes.
	 * @param o an instance of a RemoteGame
	 * @param arg an instance of Move
	 @ @see Move
	 */
	public void update(Observable o, Object arg) {
		Move move = (Move) arg;
		// showFeedBack("got an update: " + move);
		places_[move.col][move.row].setMove(move.mark);
		if (game_.notOver()) {
			showFeedBack(game_.currentPlayer().mark() + " plays");
		} else {
			showFeedBack("Game over -- " + game_.winner() + " wins!");
		}
	}
}
