package client;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;

import server.*;

/**
 * This class implements the view on a BoardGame.
 * The GameView contains a grid of Place objects that
 * intercept mouse clicks and forward them to the game.
 *<P>
 * GameView is also an Observer for the game.
 * When the game notifies the Applet of state change, it
 * updates the corresponding Place.
 *<P>
 * Below the grid there is a Label component
 * to display messages.
 *
 * <P>
 * <B>To do:</B>
 * <UL>
 * <LI>add a mechanism to select TicTacToe or Gomoku
 * <LI>provide feedback to player when a second player connects
 * <LI>visualize automated tests
 * </UL>
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public class GameView extends Frame
{
	private RemoteGameFactory gameFactory_;
	private RemoteGame game_;
	private Label label_;
	private Place[][] places_;

	/**
	 * The character of our Player.
	 * To keep the client/server interface small,
	 * we do not have a Remote interface to the
	 * game's Players, but simply represent a
	 * Player as a char ('X' or 'O').  The remote
	 * game interprets this on the server side ...
	 **/
	private char mark_;

	static private final int MINSIZE = 30;

	/**
	 * Instantiates the RemoteGame.  Initializes the view components,
	 * and connects the view (this object) to the model (RemoteGame).
	 *
	 * @param gameFactory a remote object obtained by the applet
	 **/
	public GameView(RemoteGameFactory gameFactory) {
		try {
			gameFactory_ = gameFactory;
			game_ = gameFactory_.joinGame();
			mark_ = game_.join();
			
			// Let the remote game notify us of changes:
			game_.addObserver(new GameObserver(this));
			
			this.setTitle("Player " + mark());

			setLayout(new BorderLayout());
			setSize(MINSIZE*game_.cols(),MINSIZE*game_.rows());

			this.add("North", makeControls());

			add("Center", makeGrid());

			label_ = new Label();
			add("South", label_);

			showFeedBack(game_.currentPlayer() + " plays");		
		} catch (RemoteException e) {
			error("Can't invoke remote game methods: " + e);
		}
	}

	/**
	 * @return the RemoteGame associated with this view
	 **/
	public RemoteGame game() { return game_; }

	/**
	 * @return the mark ('X' or 'O') associated with this view
	 **/
	public char mark() { return mark_; }

	/**
	 * @return a Panel holding "New Game" and "Quit" Buttons
	 **/
	private Component makeControls() {
		Panel controls = new Panel();
		controls.setLayout(new FlowLayout());

		Button again = new Button("New game");
		again.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showFeedBack("starting new game ...");
				GameView view = new GameView(gameFactory_);
				view.show();
				dispose();
			}
		});
		controls.add(again);

		Button quit = new Button("Quit");
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showFeedBack("Goodbye!");
				dispose();
			}
		});
		controls.add(quit);

		return controls;
	}

	/**
	 * @return a grid that serves as a view and controller for the game.
	 **/
	private Component makeGrid()
		throws RemoteException
	{		int cols = game_.cols();
		int rows = game_.rows();

		Panel grid = new Panel();
		
		grid.setLayout(new GridLayout(rows, cols));

		places_ = new Place[cols][rows];
		for (int row=rows-1; row>=0; row--) {
			for (int col=0; col<cols; col++) {
				Place p = new Place(col, row);
				// Attach an EventListener that will
				// forward mouse clicks to the remote game
				p.addMouseListener(new PlaceListener(p, this));
				p.setBackground(Color.white);
				grid.add(p);
				places_[col][row] = p;
			}
		}
		return grid;
	}

	/**
	 * Displays a feedback string in the associated Label component.
	 * @param msg the string to display
	 **/
	public void showFeedBack(String msg) {
		label_.setText(msg);
	}

	/**
	 * Display a serious error.
	 **/
	protected void error(String s) {
		// Perhaps we should open a Dialog?
		System.err.println(s);
		showFeedBack(s);
	}

	/**
	 * Called by the GameObserver when the game's state changes.
	 *
	 * Note that GameView is <I>not</i> a Remote object.  Since
	 * Java supports only single inheritance, GameView cannot
	 * extend both Frame and UnicastRemoteObject.  To make the
	 * GameView accessible to the server, we must introduce a
	 * delegate GameObserver which is a remote object visible to
	 * the server, and which simply passes updates to the GameView.
	 *
	 * @see client.GameObserver
	 * @see server.Move
	 *
	 * @param move an instance of Move
	 **/
	public void update(Move move)
	{
		places_[move.col][move.row].setMove(move.mark);
		try {
			if (game_.notOver()) {
				showFeedBack(game_.currentPlayer() + " plays");
			} else {
				showFeedBack("Game over -- " + game_.winner() + " wins!");
			}
		} catch (RemoteException e) {
			error("Lost connection to game during update()! " + e);
		}
	}
}
