package tictactoe;

import java.io.*;

/**
 * Manage interaction with user.  This implementation of Player
 * gets its moves from a BufferedReader.  This can either be
 * a wrapper around the standard input stream, or a wrapper
 * around a string representing a test case.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.0 1999-03-26
 **/
public class StreamPlayer extends InactivePlayer {
	private final BufferedReader in_;

	/**
	 * Constructor to specify an alternative source of moves
	 * (e.g., a test case StringReader).
	 **/
	protected StreamPlayer(char mark, BufferedReader in) {
		super(mark);
		in_ = in;
	}
	
	/**
	 * The normal contructor to use.  Just define the mark.
	 * The Player will get its input from the standard input stream.
	 **/
	public StreamPlayer(char mark) {
		// this(mark, new BufferedReader(new InputStreamReader(System.in)));
		// hack for java 1.1.8
		super(mark);
		in_ = new BufferedReader(new InputStreamReader(System.in));
	}

	/**
	 * Special constructor to make a Player that plays a fixed
	 * set of moves from a String.  Used to define test cases.
	 **/
	public StreamPlayer(char mark, String moves) {
		// this(mark, new BufferedReader(new StringReader(moves)));
		// hack for java 1.1.8
		super(mark);
		in_ = new BufferedReader(new StringReader(moves));
	}

	/**
	 * The Player makes a move by reading a line of text from
	 * the input stream, and interpreting it using chess notation,
	 * i.e., column is 'a' through 'c' and row is
	 * '1' through '3'.  The converted position is used to
	 * call BoardGame.move().
	 *
	 * @see tictactoe.GameDriver#playGame
	 **/
	public void move()
		throws IOException
	{
		String line = in_.readLine();
		if (line == null)
			throw new IOException("end of input");
		try {
			game_.move(getCol(line), getRow(line), this);
		} catch (AssertionException err) {
			System.err.println("Invalid move ignored (" + line + ")");
		}
	}

	/**
	 * Interpret the first char of a string as a letter identifying
	 * a column of a BoardGame.
	 **/
	private int getCol(String coord)
		throws AssertionException
	{
		assert(coord != null);
		assert(coord.length() > 0);
		char col = coord.charAt(0);
		assert((col>='a') && (col<='z'));
		return col - 'a';
	}
	
	/**
	 * Interpret the second and following characters of a string
	 * as a number identifying a row of a BoardGame.
	 **/
	private int getRow(String coord)
		throws AssertionException
	{
		assert(coord != null);
		assert(coord.length() > 1);
		try {
			int row = Integer.parseInt(coord.substring(1));
			return row - 1;
		} catch (NumberFormatException err) {
			throw new AssertionException(err.getMessage());
		}
	}
}
